from pip import get_installed_distributions


def check(request):
    return {
        'distribution_list': [
            {'name': i.project_name, 'version': i.version} for i in
            get_installed_distributions()
        ]
    }



import base64

from functools import wraps

from .settings import HEARTBEAT
from django.http import HttpResponse
from django.core.exceptions import ImproperlyConfigured


def http_basic_auth(func):
    @wraps(func)
    def _decorator(request, *args, **kwargs):
        prepare_auth()
        if request.META.get('HTTP_AUTHORIZATION'):
            authmeth, auth = request.META['HTTP_AUTHORIZATION'].split(' ')
            if authmeth.lower() == 'basic':
                auth = base64.b64decode(auth).decode('utf-8')
                username, password = auth.split(':')
                if (username == HEARTBEAT['auth']['username'] and
                        password == HEARTBEAT['auth']['password']):
                    return func(request, *args, **kwargs)
        response = HttpResponse(
            "your not wise enough to be in PonYLaNd 1337", status=401)
        response['WWW-Authenticate'] = 'Basic realm="Welcome to PonYLaNd 1337"'
        return response
    return _decorator


def prepare_auth():
    auth = HEARTBEAT.get('auth')
    if not auth:
        raise ImproperlyConfigured('Missing auth configuration for heartbeat')

    if not all([auth.get('username'), auth.get('password')]):
        raise ImproperlyConfigured(
            'Username or password missing from auth configuration '
            'for heartbeat')



from importlib import import_module
from django.http import HttpResponse, JsonResponse

from .settings import HEARTBEAT
from .auth import http_basic_auth


def index(request):
    return HttpResponse(content='all good in the hood')


@http_basic_auth
def details(request):
    data = {}
    for checker in HEARTBEAT['checkers']:
        checker_module = import_module(checker)
        data.update(checker_module.check(request))
    return JsonResponse(data)



# print('caca'*1000)
# import haystack
# except Exception:
#     print('dumnezaii tai' * 1000)
# from haystack.exceptions import MissingDependency
# from django.core.exceptions import ImproperlyConfigured
# from haystack.backends.elasticsearch_backend import ElasticsearchSearchBackend


def check(request):
    return {'1': 2}
    # if error:
    #     return {'elasticsearch': {'error': error}}
    # try:
    #     actual_backend = connections['default'].get_backend()
    # except (NameError, MissingDependency):
    #     actual_backend = None
    # actual_backend = default_engine.get_backend()
    # if actual_backend and not isinstance(actual_backend, ElasticsearchSearchBackend):
    #     raise ImproperlyConfigured('Elastic search is not your default')

    # try:
    #     cluster_health = actual_backend.conn.cluster.health()
    #     info = actual_backend.conn.info()
    # except (AttributeError, ConnectionError) as e:
    #     return {'elasticsearch': {'error': str(e)}}
    # except ConnectTimeoutError as e:
    #     return {'elasticsearch': {'error': str(e)}}

    # return {
    #     'elasticsearch': {
    #         'cluster_health': cluster_health,
    #         'info': info}}


# def _elasticsearch_lookup():
#     for engine in connections:
#         if isinstance(engine.get_backend(), ElasticsearchSearchBackend):
#             return engine.get_backend()



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from pkg_resources import get_distribution, DistributionNotFound


def check(request):
    package_name = settings.HEARTBEAT.get('package_name')
    if not package_name:
        raise ImproperlyConfigured(
            'Missing package_name key from heartbeat configuration')

    try:
        distro = get_distribution(package_name)
        value = '{project}=={version}'.format(
                project=distro.project_name, version=distro.version)
    except DistributionNotFound:
        value = 'no distribution found for {}'.format(package_name)

    return {'project_version': value}
